# A program which manages Ubuntu-it's web test server
# Copyright (C) 2015  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import subprocess
import os
import shutil


class Branch:
    """Representation of a branch"""

    def __init__(self, manager, name, merge_request):
        self.manager = manager

        self.name = name
        self.mr = merge_request
        self.mr_id = None
        self.mr_url = None
        self.author = None
        self.author_url = None
        self.assignee = None
        self.assignee_url = None

        self.pinned = name in manager.config["keep-branches"]

        self.check_remote_status()
        self.check_local_status()

    def check_remote_status(self):
        """Check the branch merging status on GitLab"""
        if self.pinned:
            self.active = True
            return

        result = self.manager.gitlab.merge_request(self.mr)
        if result:
            active = result["state"] == "opened"
        else:
            active = False
        self.active = active
        self.mr_id = result["iid"]
        self.mr_url = (self.manager.config["gitlab-url"]+"/"+
                       self.manager.config["gitlab-project"]+
                       "/merge_requests/"+str(self.mr_id))

        self.author = result["author"]["username"]
        self.author_url = result["author"]["web_url"]

        if result["assignee"] is not None:
            self.assignee = result["assignee"]["username"]
            self.assignee_url = result["assignee"]["web_url"]

    def check_local_status(self):
        """Check if the branch is present locally"""
        present_in = ["envs", "caches", "public"]
        for one in present_in:
            path = os.path.join(self.manager.root, one, self.name)
            if not os.path.exists(path):
                self.present = False
                return
        self.present = True

    def deploy(self):
        """Deploy the branch"""
        self.check_local_status()
        # Ok, branch already deployed
        if self.present:
            return

        make_dirs_in = ["envs", "caches", "public"]
        for dir in make_dirs_in:
            os.mkdir(os.path.join(self.manager.root, dir, self.name))

        commands = [
            "rm -rf %s" % self.manager.build_dir,
            "mkdir -p %s" % self.manager.build_dir,
            "cd %s && git fetch" % self.manager.git_dir,
            "git --git-dir=%s/.git --work-tree=%s checkout -f %s" %
                (self.manager.git_dir, self.manager.build_dir, self.name),
            "cd %s && invoke build" % self.manager.build_dir,
            "virtualenv -p python3 %s/%s" % (self.manager.envs_dir, self.name),
            "%s/%s/bin/pip install %s/build/packages/*.whl" %
                (self.manager.envs_dir, self.name, self.manager.build_dir),
            ("ln -s %s/%s/lib/python3.%s/site-packages/uitwww/static "
             "%s/%s/static" % (self.manager.envs_dir, self.name,
                sys.version_info[1], self.manager.public_dir, self.name)),
            "rm -rf %s" % self.manager.build_dir,
        ]

        for command in commands:
            subprocess.call(command, shell=True)

        self.manager.details["branches"][self.name] = self.mr
        self.manager.save_details()

    def destroy(self):
        """Destroy the local copy"""
        self.check_local_status()
        # Ok, branch not present
        if not self.present:
            return

        remove_from = ["envs", "caches", "public"]
        for dir in remove_from:
            shutil.rmtree(os.path.join(self.manager.root, dir, self.name))

        try:
            del self.manager.details["branches"][self.name]
        except KeyError:
            pass
        self.manager.save_details()
